;-----------------------------------------------------
;	ISPSwitch
;-----------------------------------------------------
;Version:	002
;Datum	:	5.10.2001
;
;zugehoerige Schaltung ist in EagleDatei ISPSwitcht.sch
;
;ueber einen Taster kann wahlweise einer von 3 Prozessoren mit 
;mit den relevanten ISP-Signalen versorgt werden (MISO,MOSI,SCK)
;oder alle werden zu einem gemeinsamen SPI-Bus zusammengeschaltet
;
;-----------------------------------------------------
;Diff02/01:	Zustand4 alle P aus hinzugefuegt
;		LED-Reihenfolge von links nach rechts angeordnet



.include "1200def.inc"

;--------------------
; Defines
;--------------------
.def	temp	= r16
.def	temp1	= r17	;nur in INT0
.def	temp2	= r18	;nur in TMR0
.def	EntprellZeitAbgelaufen   = r19
.def	Timer1	= r20
.def 	Zustand = r21


;--------------------
;* Interrupt Vectors *
;--------------------

.CSEG
	rjmp	RESET		; Reset Handle
	rjmp    EXT_INT0	; IRQ0 Handler
	rjmp	TIM0_OVF	; Timer0 Handler
	reti			; Analog Comparator Handle (not used)


;-----------------------------------
;	Int0 -Handler
;-----------------------------------

EXT_INT0:
	in	r0,SREG		; store SREG
	;Zur Entprellung den Interupt fr Timer1 * 0,025s (z.Z.0,5s) nicht auswerten
	;eventuell anstehendes Interupt Flag loeschen
	clr	temp1	; 
	out	GIMSK,temp1	; INT0 verbieten
	clr	temp1	; 
	out	MCUCR,temp1	; INT0 auf Level 0 sensibilisieren
	ldi	temp1,0x02	; 
	out	MCUCR,temp1	; INT0 auf fallende Flanke sensibilisieren
				; bei diesen 2 Schritten wurde das eventuell anhaengige INT0 Flag gelscht
	ldi	temp1,0x40	; 
	out	GIMSK,temp1	; int0 enable
	
	
	; die Freigabe der eigentlichen Verarbeitung erfolgt ber EntprellZeitAbgelaufen
	; EntprellZeitAbgelaufen=1 --> Verarbeiten 0 --> uebergehen
	ser	temp1
	cp	EntprellZeitAbgelaufen,temp1
	brne	ENDE
	;EntprellZeit starten, dazu TMR0 starten
	clr	EntprellZeitAbgelaufen
	ldi	temp1,(256-180)	; 0,0001388*180=0,025s
	out	TCNT0,temp1
	ldi	Timer1,20	;20*0,025s = 0,5s Entprellzeit	; 
	ldi 	temp1,0x02
	out 	TIMSK,temp1	; enable Timer0 interupt
	
	;Tastenwert aendern und passende StatusLed bewegen
	;Zustand = 0 P1..3 an
	;	 = 1 nur P1 ein	
	;	 = 2 nur P2 ein	
	;	 = 3 nur P3 ein	
	;	 = 4 P1..3 aus, zum LED-Stromsparen hehe
	; Taster wurde gedrckt, also Zustand erhhen, nach 4 kommt 0 hehe
	;4053 disablen
	sbi	PORTB,0
	
	inc 	Zustand;
	ldi	temp1,5
	cp	Zustand,temp1
	brne	V0	;aus 5 wird 0
	clr	Zustand
V0:			;0?
	ldi	temp1,0
	cp	Zustand,temp1
	brne	V1	;
	;Schalter einstellen
	cbi	PORTB,1	;P1 ein vorbereiten
	cbi	PORTB,2	;P2 ein vorbereiten
	cbi	PORTB,3	;P3 ein vorbereiten
	cbi	PORTD,0 ;LED0 an
	cbi	PORTD,1 ;LED1 an 
	cbi	PORTD,3 ;LED2 an
	rjmp	AUSG 
	
V1:			;nur P1 ein
	ldi	temp1,1
	cp	Zustand,temp1
	brne	V2	;
	;Schalter einstellen
	cbi	PORTB,1	;P1 ein vorbereiten
	sbi	PORTB,2	;P2 aus vorbereiten
	sbi	PORTB,3	;P3 aus vorbereiten
	;Reihenfolge nach Lage auf Platine von links nach rechts angepasst
	sbi	PORTD,0 ;LED0 aus
	sbi	PORTD,1 ;LED1 aus 
	cbi	PORTD,3 ;LED2 an -->P1
	rjmp	AUSG 
	
V2:			;nur P2 ein
	ldi	temp1,2
	cp	Zustand,temp1
	brne	V3	;
	;Schalter einstellen
	sbi	PORTB,1	;P1 aus vorbereiten
	cbi	PORTB,2	;P2 ein vorbereiten
	sbi	PORTB,3	;P3 aus vorbereiten
	sbi	PORTD,0 ;LED0 aus
	cbi	PORTD,1 ;LED1 an 
	sbi	PORTD,3 ;LED2 aus
	rjmp	AUSG 
V3:			;nur P3 ein
	ldi	temp1,3
	cp	Zustand,temp1
	brne	V4	
	;Schalter einstellen
	sbi	PORTB,1	;P1 aus vorbereiten
	sbi	PORTB,2	;P2 aus vorbereiten
	cbi	PORTB,3	;P3 ein vorbereiten
	;Reihenfolge nach Lage auf Platine von links nach rechts angepasst
	cbi	PORTD,0 ;LED0 an   -->P3	
	sbi	PORTD,1 ;LED1 aus 
	sbi	PORTD,3 ;LED2 aus
	rjmp	AUSG 
	
V4:			;4?
	ldi	temp1,4
	cp	Zustand,temp1
	brne	ENDE	;hier duerften wir nie abspringen, 
			;das Ergebnis waere gleich, weil
			;/EN uebersprungen wird, 
			;sind alle Prozessoren weg vom Bus, 
			;aber die LEDs leuchten noch !!!
	;Schalter einstellen, koennte man auch nur mit /EN machen
	sbi	PORTB,1	;P1 aus vorbereiten
	sbi	PORTB,2	;P2 aus vorbereiten
	sbi	PORTB,3	;P3 us vorbereiten
	sbi	PORTD,0 ;LED0 aus
	sbi	PORTD,1 ;LED1 aus
	sbi	PORTD,3 ;LED2 aus
	rjmp	AUSG 
	
AUSG:	;4053 enablen
	cbi	PORTB,0
	
	
ENDE:
	out	SREG,r0	        ; restore SREG
	reti

;-------------------------------------------
;	Timer0 Overlfow - Handler
;-------------------------------------------

TIM0_OVF:
	in	r0,SREG		; store SREG
	ldi	temp2,(256-180)	; 0,0001388*180=0,025s
	out	TCNT0,temp2	;
	dec	Timer1
	; Pruefen ob die Anzahl der Durchlauefe aus Timer1 schon stimmt 
	brne	ENDE1
	  
	ser     EntprellZeitAbgelaufen 			; 
	clr 	temp2
	out 	TIMSK,temp2	; disable Timer0 interupt, Timer wird in INT0 gestartet
ENDE1:	
	out	SREG,r0	        ; restore SREG
	reti			; exit

;---------------------------------------------------------
;
; RESET -->. Initialisierung 
;
;---------------------------------------------------------

RESET:	
	clr 	temp
	out 	TIMSK,temp	; disable Timer0 interupt
	ldi	temp,0x05	; CK/1024 tickt den TMR0 d.h. 1,388*e-4 = 0,0001388 sek bei 7,3728MHz
	out	TCCR0,temp
	ldi	temp,0x0b	; PD0..1,PD3 out, PD2,PD4..7 in	
	out	DDRD,temp	; 
	ldi	temp,0xfb	; PD2=INT0 auslassen
	out 	PORTD,temp	; bei den Inputs werden die Pullups enabled, die Ouputs gehen auf HI
				; d.h. /EN und /Pn_EIN HI-> ISP Bus ist getrennt	
	ldi	temp,0x0f	; PB0..PB3 out, PB4..7 in		
	out	DDRB,temp	; 
	ser	temp	
	out 	PORTB,temp	; bei den Inputs werden die Pullups enabled, die Ouputs gehen auf HI
				; d.h. LED0..2 sind aus	
	ser	EntprellZeitAbgelaufen
	; ISP Bus einschalten
	;4053 disablen
	sbi	PORTB,0
	;Schalter einstellen
	cbi	PORTB,1	;P1 ein vorbereiten
	cbi	PORTB,2	;P2 ein vorbereiten
	cbi	PORTB,3	;P3 ein vorbereiten
	clr	Zustand	; alles an ist Zustand 0
	cbi	PORTB,0	;und gueltig machen
	
	cbi	PORTD,0 ;LED0 an
	cbi	PORTD,1 ;LED1 an 
	cbi	PORTD,3 ;LED2 an 
	
	;eventuell anstehendes Interupt Flag loeschen
	clr	temp	; 
	out	GIMSK,temp	; INT0 verbieten
	clr	temp 		; 
	out	MCUCR,temp	; INT0 auf Level 0 sensibilisieren
	ldi	temp,0x02	; 
	out	MCUCR,temp	; INT0 auf fallende Flanke sensibilisieren
				; bei diesen 2 Schritten wurde das eventuell anhaengige INT0 Flag gelscht
	ldi	temp,0x40	; 
	out	GIMSK,temp	; int0 enable
	
	sei			; Enable interrupts
	rjmp 	MAIN


MAIN:

LOOP:
	nop
	rjmp 	LOOP